//
// Pageman - HTML, Markdown and Legend mixer that Loves Jekyll...
//    Copyright 2016 by Pouya Kary <k@karyfoundation.org>
//
"use strict";
//
// ─── INCLUDES ───────────────────────────────────────────────────────────────────
//
const marked = require('marked');
//
// ─── GLOBAL STORAGE ─────────────────────────────────────────────────────────────
//
var resultParagraphs = new Array();
var currentLegend = 1;
var ids = {};
//
// ─── CONSTANTS ──────────────────────────────────────────────────────────────────
//
/**
 * The grammar fo the legend
 */
let legendGrammar = /\^([^\^])+\^/gi;
//
// ─── COMPILE ────────────────────────────────────────────────────────────────────
//
/**
 * Compiles a legend string code into HTML
 */
function compile(code, startingIndex) {
    // init
    resultParagraphs = Array();
    currentLegend = startingIndex;
    // compile
    getParagraphs(code).forEach(paragraph => {
        resultParagraphs.push(compileParagraph(paragraph));
    });
    // render 
    let renderedCode = renderParagraphTableRowArray();
    // done
    return { compiledSource: renderedCode, currentLegendNumber: currentLegend };
}
exports.compile = compile;
//
// ─── LOADING RESOURCES ──────────────────────────────────────────────────────────
//
function completeHTML(code) {
    return `<link rel="stylesheet" href="legend.css">\n${code}`;
}
exports.completeHTML = completeHTML;
// ────────────────────────────────────────────────────────────────────────────────
//
// ──────────────────────────────────────────────────────── II ──────────
//  :::::: C O M P I L E R : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────
//
//
// ─── COMPILE PARAGRAPH ──────────────────────────────────────────────────────────
//
/**
 * Compiles the legend part of the code as well as the markdown paragraphs
 */
function compileParagraph(paragraphCode) {
    // our result
    var result = {
        paragraph: '',
        legends: new Array()
    };
    // compiling
    result.paragraph = paragraphCode.replace(legendGrammar, (match) => {
        // creating the id
        let id = createUniqueId();
        // adding the legend
        let legendContent = marked(match.substring(1, match.length - 1));
        // removing <p> ... </p>
        legendContent = legendContent.substring(3, legendContent.length - 5);
        // adding it...
        result.legends.push(`<div class="legend-bullet" id="${id}">${currentLegend} &bull; ${legendContent}</div>`);
        //returning the id
        return `<sup class="legend-sup"><a href="#${id}">&dagger;${currentLegend++}</a></sup>`;
    });
    // compiling markdown
    result.paragraph = marked(result.paragraph);
    // done
    return result;
}
//
// ─── GENERATE RANDOM IDS ────────────────────────────────────────────────────────
//
function createUniqueId() {
    while (true) {
        let id = `legend${Math.random().toString(16).slice(2)}`;
        if (ids[id] === undefined) {
            ids[id] = true;
            return id;
        }
    }
}
//
// ─── GET PARAGRAPHS ─────────────────────────────────────────────────────────────
//
function getParagraphs(code) {
    return code.split('\n\n');
}
// ────────────────────────────────────────────────────────────────────────────────
//
// ──────────────────────────────────────────────────────── III ──────────
//  :::::: R E N D E R E R : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────
//
//
// ─── RENDERING THE TEXT ─────────────────────────────────────────────────────────
//
/**
 * Renders the **`paragraphTableRow[ ]`**
 */
function renderParagraphTableRowArray() {
    var result = new Array();
    resultParagraphs.forEach(row => {
        result.push(renderParagraph(row));
    });
    return result.join('\n\n');
}
//
// ─── RENDER PARAGRAPH ───────────────────────────────────────────────────────────
//
function renderParagraph(row) {
    return ('<div class="legend-section">' +
        `<div class="legend-paragraph">${row.paragraph}</div>` +
        `<div class="legend-sidebar"><ul>${row.legends.join('\n')}</ul></div>` +
        '</div>');
}
// ────────────────────────────────────────────────────────────────────────────────
