//
// Pageman - HTML, Markdown and Legend mixer that Loves Jekyll and Kary Comments.
//    Copyright 2016 by Pouya Kary <k@karyfoundation.org>
//
"use strict";
//
// ─── INCLUDES ───────────────────────────────────────────────────────────────────
//
const legend = require('./legend');
const marked = require('marked');
//
// ─── ENUMS ──────────────────────────────────────────────────────────────────────
//
var language;
(function (language) {
    language[language["html"] = 0] = "html";
    language[language["markdown"] = 1] = "markdown";
    language[language["legend"] = 2] = "legend";
})(language || (language = {}));
//
// ─── GLOBAL SETTINGS ────────────────────────────────────────────────────────────
//
var currentLegendNumber = 1;
var resetLegendAtPart = false;
//
// ─── START ──────────────────────────────────────────────────────────────────────
//
function compile(pageText) {
    let parsedParts = parseParts(pageText);
    return compileParts(parsedParts);
}
exports.compile = compile;
//
// ─── PARSE PARTS ────────────────────────────────────────────────────────────────
//
/**
 * Parses the text file and detects the ***language parts***.
 */
function parseParts(pageText) {
    // defs
    var result = new Array();
    var currentLanguage = language.html;
    var currentValueLines = new Array();
    // functions
    function pushCleanReset() {
        if (currentValueLines.length > 0) {
            result.push({
                kind: currentLanguage,
                value: currentValueLines.join('\n')
            });
            currentLanguage = language.html;
            currentValueLines = [];
        }
    }
    // body
    pageText.split('\n').forEach(line => {
        // if not comment
        if (!line.match(/^\s*\/\/.*$/gi)) {
            // starting markdown part
            if (line.match(/^\s*---\s*(md|mdown|markdown)\s*$/i)) {
                pushCleanReset();
                currentLanguage = language.markdown;
            }
            else if (line.match(/^\s*---\s*legend\s*$/i)) {
                pushCleanReset();
                currentLanguage = language.legend;
            }
            else if (line.match(/^\s*---\s*(e|end)\s*$/i)) {
                pushCleanReset();
            }
            else {
                currentValueLines.push(line);
            }
        }
    });
    // Adding last results
    pushCleanReset();
    // done
    return result;
}
exports.parseParts = parseParts;
//
// ─── COMPILE PARTS ──────────────────────────────────────────────────────────────
//
/**
 * Compiles different page parts
 */
function compileParts(parts) {
    let lines = new Array();
    parts.forEach(part => {
        switch (part.kind) {
            case language.legend:
                let legendResult = legend.compile(part.value, currentLegendNumber);
                if (!resetLegendAtPart) {
                    currentLegendNumber = legendResult.currentLegendNumber;
                }
                lines.push(legendResult.compiledSource);
                break;
            case language.markdown:
                lines.push(marked(part.value));
                break;
            default:
                lines.push(part.value);
                break;
        }
    });
    // and done...
    return lines.join('\n');
}
// ──────────────────────────────────────────────────────────────────────────────── 
